comment @ ********************************

        The "txtbutn" control is a transparent window that displays only
        the text assigned to the control. It differs from a normal button
        in that it sends a WM_COMMAND message on both its UP and DOWN
        click so that it can be used for both mouse click directions.

        It has two extra arguments, a class name and a cursor so that each
        control can have a different cursor if required and the control
        works correctly on both text and symbols from the specialised
        fonts like Wingdings and Webdings.

        The control has a separate structure for settings that tend to be
        common across a number of grouped buttons, font, alignment and
        text colour. The shadow on the text is preset to black for both
        UP and DOWN display. The control has been tested on coloured client
        areas, bitmap backgrounds set in STATIC controls and appears to
        display correctly on both.

        The alignment of the text is controlled by 1 or 3 values,
        -1 = LEFT align
        0  = CENTRE align
        1  = RIGHT align
        If a value outside this range is used the control defaults to 
        CENTRE alignment.

        ******************************** @

    .486                      ; create 32 bit code
    .model flat, stdcall      ; 32 bit memory model
    option casemap :none      ; case sensitive

    include txtbutn.inc        ; local includes for this file

    include lib\tbutn.inc
    includelib lib\tbutn.lib

    TBUTNSTRUCT STRUCT
      txtcolor dd ?
      txtalign dd ?
      fonthndl dd ?
    TBUTNSTRUCT ENDS

    .data?
      hTxtFont dd ?
      hWindng2 dd ?
      hWindng3 dd ?
      hWebdngs dd ?
      butn1 dd ?
      butn2 dd ?
      butn3 dd ?
      butn4 dd ?

.code

start:

; 

      invoke InitCommonControls

    ; ------------------
    ; set global values
    ; ------------------
      mov hInstance,   rv(GetModuleHandle, NULL)
      mov CommandLine, rv(GetCommandLine)
      mov hIcon,       rv(LoadIcon,hInstance,500)
      mov hCursor,     rv(LoadCursor,NULL,IDC_ARROW)
      mov sWid,        rv(GetSystemMetrics,SM_CXSCREEN)
      mov sHgt,        rv(GetSystemMetrics,SM_CYSCREEN)

      call Main

      invoke ExitProcess,eax

; 

Main proc

    LOCAL Wwd:DWORD,Wht:DWORD,Wtx:DWORD,Wty:DWORD,hBrush:DWORD

    STRING szClassName,"Demo_Class"

  ; --------------------------------------------
  ; register class name for CreateWindowEx call
  ; --------------------------------------------

    mov hBrush, rv(CreateSolidBrush,00FF7777h)


    invoke RegisterWinClass,ADDR WndProc,ADDR szClassName,
                       hIcon,hCursor,hBrush

  ; -------------------------------------------------
  ; macro to autoscale window co-ordinates to screen
  ; percentages and centre window at those sizes.
  ; -------------------------------------------------
    AutoScale 40, 30

    invoke CreateWindowEx,WS_EX_LEFT,
                          ADDR szClassName,
                          ADDR szDisplayName,
                          WS_OVERLAPPEDWINDOW,
                          Wtx,Wty,Wwd,Wht,
                          NULL,NULL,
                          hInstance,NULL
    mov hWnd,eax

  ; ---------------------------
  ; macros for unchanging code
  ; ---------------------------
    DisplayWindow hWnd,SW_SHOWNORMAL

    call MsgLoop
    ret

Main endp

; 

RegisterWinClass proc lpWndProc:DWORD, lpClassName:DWORD,
                      Icon:DWORD, Cursor:DWORD, bColor:DWORD

    LOCAL wc:WNDCLASSEX

    mov wc.cbSize,         sizeof WNDCLASSEX
    mov wc.style,          CS_BYTEALIGNCLIENT or \
                           CS_BYTEALIGNWINDOW
    m2m wc.lpfnWndProc,    lpWndProc
    mov wc.cbClsExtra,     NULL
    mov wc.cbWndExtra,     NULL
    m2m wc.hInstance,      hInstance
    m2m wc.hbrBackground,  bColor
    mov wc.lpszMenuName,   NULL
    m2m wc.lpszClassName,  lpClassName
    m2m wc.hIcon,          Icon
    m2m wc.hCursor,        Cursor
    m2m wc.hIconSm,        Icon

    invoke RegisterClassEx, ADDR wc

    ret

RegisterWinClass endp

; 

MsgLoop proc

    LOCAL msg:MSG

    push esi
    push edi
    xor edi, edi                        ; clear EDI
    lea esi, msg                        ; Structure address in ESI
    jmp jumpin

    StartLoop:
      invoke TranslateMessage, esi
      invoke DispatchMessage,  esi
    jumpin:
      invoke GetMessage,esi,edi,edi,edi
      test eax, eax
      jnz StartLoop

    mov eax, msg.wParam
    pop edi
    pop esi

    ret

MsgLoop endp

; 

WndProc proc hWin:DWORD,uMsg:DWORD,wParam:DWORD,lParam:DWORD

    LOCAL var    :DWORD
    LOCAL caW    :DWORD
    LOCAL caH    :DWORD
    LOCAL Rct    :RECT
    LOCAL buffer1[260]:BYTE  ; these are two spare buffers
    LOCAL buffer2[260]:BYTE  ; for text manipulation etc..

    LOCAL cstruct  :TBUTNSTRUCT

    Switch uMsg
      Case WM_COMMAND
        switch wParam
          case 500
            switch lParam
              case 0
                fn SetWindowText,hWin,"Up"
              case 1
                fn SetWindowText,hWin,"Down"
            endsw

          case 501
            switch lParam
              case 0
                fn SetWindowText,hWin,"Up"
              case 1
                fn SetWindowText,hWin,"Down"
            endsw

          case 502
            switch lParam
              case 0
                fn SetWindowText,hWin,"Up"
              case 1
                fn SetWindowText,hWin,"Down"
            endsw

          case 503
            switch lParam
              case 0
                fn SendMessage,hWin,WM_SYSCOMMAND,SC_CLOSE,0
              case 1
                fn SetWindowText,hWin,"Bye"
            endsw

        endsw

      Case WM_CREATE
        invoke Do_Status,hWin

        mov hTxtFont, GetFontHandle("times new roman",20,700)
        mov hWindng2, GetFontHandle("wingdings 2",20,700)
        mov hWindng3, GetFontHandle("wingdings 3",20,700)
        mov hWebdngs, GetFontHandle("webdings",26,500)

        mov cstruct.txtcolor, 00FFFFFFh
        mov cstruct.txtalign, 1
        mrm cstruct.fonthndl, hTxtFont

        fn txtbutn,"&New File ",hInstance,hWin,"b1class",hCursor,100,10,100,22,500,ADDR cstruct
        fn txtbutn,"&Open File ",hInstance,hWin,"b2class",hCursor,100,35,100,22,501,ADDR cstruct
        fn txtbutn,"&Save File ",hInstance,hWin,"b3class",hCursor,100,60,100,22,502,ADDR cstruct
        fn txtbutn,"&Close ",hInstance,hWin,"b4class",hCursor,100,85,100,22,503,ADDR cstruct

        mov cstruct.txtalign, 0
        mrm cstruct.fonthndl, hWindng3
      ; --------------------------------------------
      ; single characters are winding 3 font symbols
      ; The displayed characters are direction arrows
      ; --------------------------------------------
        fn txtbutn,"p",hInstance,hWin,"upclass",hCursor,10,110,25,25,504,ADDR cstruct
        fn txtbutn,"q",hInstance,hWin,"downclass",hCursor,40,110,25,25,505,ADDR cstruct
        fn txtbutn,"t",hInstance,hWin,"leftclass",hCursor,70,110,25,25,506,ADDR cstruct
        fn txtbutn,"u",hInstance,hWin,"rightclass",hCursor,100,110,25,25,507,ADDR cstruct

        mov cstruct.txtalign, 0
        mrm cstruct.fonthndl, hWebdngs
      ; --------------------------------------------
      ; single characters are webdings font symbols
      ; These displayed characters are for music or
      ; video player controls.
      ; --------------------------------------------
        fn txtbutn,"4",hInstance,hWin,"mp1",hCursor,10,140,25,25,510,ADDR cstruct
        fn txtbutn,"8",hInstance,hWin,"mp2",hCursor,40,140,25,25,511,ADDR cstruct
        fn txtbutn,":",hInstance,hWin,"mp3",hCursor,70,140,25,25,512,ADDR cstruct
        fn txtbutn,cfm$("\l"),hInstance,hWin,"mp4",hCursor,100,140,25,25,513,ADDR cstruct
        fn txtbutn,";",hInstance,hWin,"mp5",hCursor,130,140,25,25,513,ADDR cstruct
        fn txtbutn,"9",hInstance,hWin,"mp6",hCursor,160,140,25,25,514,ADDR cstruct
        fn txtbutn,"7",hInstance,hWin,"mp7",hCursor,190,140,25,25,515,ADDR cstruct
        fn txtbutn,"3",hInstance,hWin,"mp8",hCursor,220,140,25,25,516,ADDR cstruct

      Case WM_SYSCOLORCHANGE

      Case WM_SIZE
        invoke MoveWindow,hStatus,0,0,0,0,TRUE

      Case WM_CLOSE
        invoke DeleteObject,hTxtFont
        invoke DeleteObject,hWindng2
        invoke DeleteObject,hWindng3
        invoke DeleteObject,hWebdngs

      Case WM_DESTROY
        invoke PostQuitMessage,NULL
        return 0

    Endsw

    invoke DefWindowProc,hWin,uMsg,wParam,lParam

    ret

WndProc endp

; 

OPTION PROLOGUE:NONE 
OPTION EPILOGUE:NONE 

TopXY proc wDim:DWORD, sDim:DWORD

    mov eax, [esp+8]
    sub eax, [esp+4]
    shr eax, 1

    ret 8

TopXY endp

OPTION PROLOGUE:PrologueDef 
OPTION EPILOGUE:EpilogueDef 

; 

end start
